/**
* Creates an instance of Widget
*
* @extends {Modeable}
* @class
* @name Widget
* @param {ScreenService} ScreenService - An instance of ScreenService.
* @param {ZhUtil} ZhUtil - An instance of ZhUtil.
* @param {EventEngine} eventEngine - An instance of EventEngine.
* @param {ZHPromise} ZHPromise - An instance of ZHPromise.
* @param {TemplateManager} templateManager - An instance of TemplateManager
* @param {ZhLogger} ZhLogger - An instance of ZhLogger.
* @param {ZhFeatures} ZhFeatures - An instance of ZhFeatures
*/
var Widget = function(ScreenService, ZhUtil, eventEngine, ZHPromise, templateManager, ZhLogger, ZhFeatures) {
	var self = this;

	var emptyFunction = function() {};

	var FORM_TEMPLATE = 'widget/form.html';
	var FORM_TEMPLATE_VIEW = 'widget/form-view.html';
	var DATASOURCE_FILTER_PROPERTY = 'dataSourceFilter';

	/**
	* Default values of a widget
	* @var {Object} DEFAULT_VALUES
	* @memberof Widget
	* @instance
	*/
	var DEFAULT_VALUES = {
		showDescriptionOrder: true,
		selectDetailOnMoveRow: true, // Master Detail
		allowMultiSelect: false,
		allowSelectOnLongTap: true,
		parentFilter: {
			refkeys: [],
			parentkeys: []
		},
		actions: [],
		events: [],
		editing: false,
		active: false,
		templateToView: FORM_TEMPLATE_VIEW,
		templateToForm: FORM_TEMPLATE,
		formTemplate: FORM_TEMPLATE,
		defaultChildTemplate: FORM_TEMPLATE_VIEW,
		aclIsVisible: true,
		aclReadOnly: false,
		currentRow: {},
		selectedRow: {},
		itemsPerPage: 500,
		mode: ViewMode.LIST,
		searchWithPagination: true,
		hasPagination: true,
		showAsTab: true,
		dataSourceFilter: [],
		fields: [],
		fieldSequence: 0,
		dataSource: {},
		selectCascade: false,
		childrenDependency: false,
		redraw: emptyFunction,
		skipSearch: false,
		enabled: true,
		reportShowOnlyVisibleFields: true,
		orderFieldsBySequenceInForm: false,
		reloadOnEnterTab: true,
		showNeverActionsAsGrouped: false,
		applyDefaultPreference: false,
		bringGroupsCollapsed: false,
		orderFromGroups: true,
		rowHeight: 64
	};

	/**
	* Returns the default values of a widget
	* @function getDefaultValues
	* @memberof Widget
	* @instance
	* @returns {Object} Returns the default values of a widget. See {@link DEFAULT_VALUES} on {@link Widget}
	*/
	this.getDefaultValues = function() {
		return DEFAULT_VALUES;
	};

	/**
	* Gets the next field of the sequence
	* @function getNextFieldSequence
	* @memberof Widget
	* @instance
	* @returns {Number} Next field sequence
	*/
	this.getNextFieldSequence = function() {
		return ++this.fieldSequence;
	};

	/**
	* Verifies if the current row is new
	* @function isNewRow
	* @memberof Widget
	* @instance
	* @returns {Boolean} True if current row is new
	*/
	this.isNewRow = function() {
		return !Util.isUndefinedOrNull(this.currentRow) &&
			!Util.isUndefinedOrNull(this.currentRow.__is_new) &&
			this.currentRow.__is_new;
	};

	/**
	* Verifies if the widget has the LazyLoad property set as true
	* @function isLazyLoad
	* @memberof Widget
	* @instance
	* @returns {Boolean} True if Widget has LazyLoad as true
	*/
	this.isLazyLoad = function() {
		var isLazyLoad = false;
		if (this.dataSource) {
			isLazyLoad = this.dataSource.lazyLoad;
		}
		return isLazyLoad;
	};
	/**
	* Verifies if the widget is the last widget active
	* @function isLastActiveWidget
	* @memberof Widget
	* @instance
	* @returns {Boolean} True if the widget is the last one active
	*/
	this.isLastActiveWidget = function() {
		return this.container && this === this.container.getActiveWidget().lastActiveWidget;
	};

	function setParentCurrentWidget() {
		if (self.parent && self.showAsTab) {
			self.parent.currentWidget = self;
		}
	}

	function publishOnActivateEvent(isReactivating) {
		if (Util.isFunction(self.onActivate) && !isReactivating) {
			self.onActivate();
		}
	}

	function setLastActiveWidget() {
		if (self.container && (!self.isLastActiveWidget() || !self.lastActiveWidget)) {
			self.lastActiveWidget = self.container.getActiveWidget();
		}
	}

	function deactivateOtherWidgets() {
		if (self.container) {
			self.container.inactiveWidgets(self);
		}
	}

	function setModeToContainer() {
		if (self.container) {
			self.container.changeMode(self.mode);
		}
	}

	/**
	* Activates the widget
	* @function activate
	* @memberof Widget
	* @instance
	* @returns {Promise} Returns a promise after widget is activated
	*/
	this.activate = function() {
		var isReactivating = this.isActive();
		setLastActiveWidget();
		deactivateOtherWidgets();
		setModeToContainer();
		setParentCurrentWidget();
		this.active = true;
		publishOnActivateEvent(isReactivating);
		this.updateCheckedRows();
		return ZHPromise.when();
	};

	/**
	* Verifies if widget is active
	* @function isActive
	* @memberof Widget
	* @instance
	* @returns {Boolean} True if widget is active
	*/
	this.isActive = function() {
		return !!this.active;
	};

	/**
	* Deactivates the widget
	* @function deactivate
	* @memberof Widget
	* @instance
	*/
	this.deactivate = function() {
		var isActive = this.isActive();
		this.active = false;
		if (isActive && Util.isFunction(this.onDeactivate)) {
			this.onDeactivate();
		}
	};

	/**
	* Verifies if this widget is parent of any active widgets
	* @function hasActive
	* @memberof Widget
	* @instance
	* @returns {Boolean} True if widget has active widgets
	*/
	this.hasActive = function() {
		var active = this.active;
		if (!active && this.widgets && this.widgets.length > 0) {
			for (var i in this.widgets) {
				active = this.widgets[i].hasActive();
				if (active) {
					break;
				}
			}
		}
		return active;
	};

	/**
	* Restores the last widget active
	* @function restoreLastActiveWidget
	* @memberof Widget
	* @instance
	*/
	this.restoreLastActiveWidget = function() {
		if (this.lastActiveWidget) {
			this.lastActiveWidget.activate();
		}
		delete this.lastActiveWidget;
	};

	/**
	* Verifies if this widget is enabled
	* @function isEnabled
	* @memberof Widget
	* @instance
	* @returns {Boolean} True if widget is enabled
	*/
	this.isEnabled = function() {
		return !!this.enabled;
	};

	/**
	* Enables / Disbales widget
	* @function setEnabled
	* @memberof Widget
	* @instance
	* @param {Boolean} val - True to enable widget
	*/
	this.setEnabled = function(val) {
		if (this.enabled !== val) {
			this.enabled = val;
			templateManager.updateTemplate();
		}
	};

	/**
	* Changes to the first row of the widget
	* @function moveToFirst
	* @memberof Widget
	* @instance
	*/
	this.moveToFirst = function() {
		var row = this.dataSource.getFirstRow();
		if (row) {
			this.setCurrentRow(row);
		}
	};

	/**
	* Changes to the last row of the widget
	* @function moveToLast
	* @memberof Widget
	* @instance
	*/
	this.moveToLast = function() {
		var row = this.dataSource.getLastRow();
		if (row) {
			this.setCurrentRow(row);
		}
	};

	/**
	* Changes to the next row of the widget
	* @function moveToNext
	* @memberof Widget
	* @instance
	*/
	this.moveToNext = function() {
		var __index = this.dataSource.data.indexOf(this.currentRow);
		__index++;
		if (__index === this.dataSource.data.length && !this.isStripeLimited) {
			__index = 0;
		}
		if (__index < this.dataSource.data.length) {
			this.setCurrentRow(this.dataSource.data[__index]);
		}
	};

	/**
	* Changes to the previous row of the widget
	* @function moveToPrev
	* @memberof Widget
	* @instance
	*/
	this.moveToPrev = function() {
		var __index = this.dataSource.data.indexOf(this.currentRow);
		__index--;
		if (__index === -1 && !this.isStripeLimited) {
			__index = this.dataSource.data.length - 1;
		}
		if (__index >= 0) {
			this.setCurrentRow(this.dataSource.data[__index]);
		}
	};

	/**
	* Get the parent widget
	* @function getParent
	* @memberof Widget
	* @instance
	* @returns {Widget} Parent Widget
	*/
	this.getParent = function() {
		return this.parent ? this.parent : this.container;
	};

	/**
	* Changes widget to list mode
	* @function list
	* @memberof Widget
	* @instance
	*/
	this.list = function(closeSwipe) {
		this.showForm = false;
		this.editing = false;
		this.formTemplate = this.templateToView;
		_changeMode(ViewMode.LIST);
		if (closeSwipe) {
			ScreenService.closeSwipe();
		}
	};
	/**
	* Changes widget to view mode
	* @function view
	* @memberof Widget
	* @instance
	*/
	this.view = function() {
		this.showForm = true;
		this.editing = false;
		this.formTemplate = this.templateToView;
		_changeMode(ViewMode.VIEW);
	};
	
	var _changeMode = (function(mode) {
		this.container.changeMode(mode);
		this.changeMode(mode);
	}).bind(this);
	
	this._changeMode = _changeMode;

	/**
	* Changes widget to edit mode
	* @function edit
	* @memberof Widget
	* @instance
	*/
	this.edit = function() {
		this.showForm = true;
		this.editing = true;
		this.formTemplate = this.templateToForm;
		_changeMode(ViewMode.EDIT);
	};

	/**
	 * Changes widget to inline edit mode
	 *
	 * @function inlineEdit
	 * @memberOf Widget
	 * @instance
	 */
	this.inlineEdit = function() {
		this.showForm = true;
		this.editing = true;
		_changeMode(ViewMode.INLINE_EDIT);
	};

	/**
	* Get the filters on the data source of the widget
	* @function getDataSourceFilter
	* @memberof Widget
	* @instance
	* @param {String} property - (Optional) Filter property
	* @returns {Object[]} Filters of the data source
	*/
	this.getDataSourceFilter = function(property) {
		var result = [];
		if (!property || !this[property]) {
			property = DATASOURCE_FILTER_PROPERTY;
		}

		if (Util.isArray(this[property])) {
			result = Util.clone(this[property]).map(function(item) {
				var tempItem = Util.clone(item);
				tempItem.value = ZhUtil.evaluate(tempItem.value);
				return tempItem;
			});
		}
		return result;
	};

	var refreshCurrentRow = function(additionalFilters) {
		var currentRow = self.currentRow;
		var dataSource = self.dataSource;

		var VO = dataSource.getVO();

		if (!VO || !VO.localKey || !VO.localKey.length) {
			return ZHPromise.when();
		} else {
			var localKey = VO.localKey;
			var filters = additionalFilters || [];
			localKey.forEach(function eachLocalKey(key) {
				filters.push({
					"name": key,
					"operator": "=",
					"value": currentRow[key]
				});
			});
			return dataSource.filter(filters).then(function overrideCurrentRow(data) {
				var currentRow = data.shift();
				for (var item in currentRow) {
					self.currentRow[item] = currentRow[item];
				}
			});
		}
	};

	/**
	* Reloads the widget
	* @function reload
	* @memberof Widget
	* @instance
	* @param {Object[]} additionalFilters - (Optional) Filters to be applied on reload
	* @param {Boolean} shouldRefreshCurrentRow - (Optional) True if should refresh current row
	* @returns {Promise} Promise to reload the data source
	*/
	this.reload = function(additionalFilters, shouldRefreshCurrentRow) {
		if (this.dataSource) {
			if (this.mode == ViewMode.VIEW && !Util.isFalse(shouldRefreshCurrentRow)) {
				return refreshCurrentRow(additionalFilters);
			} else {
				var callback = this.container ? this.container.refresh : undefined;
				var filters = this.getDataSourceFilter();
				if (Array.isArray(additionalFilters)) {
					filters = filters.concat(additionalFilters);
				}
				this.dataSource.setCurrentPage(1);
				return this.dataSource.load(filters, this.dataSource.getCurrentPage(), this.itemsPerPage,
					callback);
			}
		} else {
			return {
				then: function(fullfil) {
					return fullfil ? fullfil() : this;
				}
			};
		}
	};
	/**
	* Load data for next page of the widget
	* @function nextPage
	* @memberof Widget
	* @instance
	*/
	this.nextPage = function() {
		if (this.dataSource) {
			this.dataSource.load(this.getDataSourceFilter(), this.dataSource.getNextPage(), this.itemsPerPage);
		}
	};
	/**
	* Register if order method is being called
	* @function order
	* @memberof Widget
	* @instance
	*/
	this.order = function order() {
		ZhFeatures.registerFeatureUsage({
			"model": "Widget",
			"widgetName": self.name
		});
	};
	/**
	* Saves the current row
	* @function saveRow
	* @memberof Widget
	* @instance
	* @returns {Promise} Returns a promise that will fullfill when data is saved
	*/
	this.saveRow = function saveRow() {
		if (this.currentRow) {
			this.currentRow.changed = true;
			if (this.dataSource.data.indexOf(this.currentRow) === -1) {
				this.dataSource.data.push(this.currentRow);
			}
			return this.dataSource.save();
		} else {
			throw new Error('Current row is undefined. Unable to save row.');
		}
	};

	/**
	* Create a object with all the values of the fields
	* @function getReapeatOnNewValues
	* @memberof Widget
	* @instance
	* @returns {Object} Values of the fields
	*/
	this.getRepeatOnNewValues = function() {
		return this.fields.reduce(function(row, field) {
			var value = field.value();
			if (field.repeatOnNew && value) {
				row[field.name] = value;
			}
			return row;
		}, {});
	};

	/**
	* Creates a new row on the widget to insert new values
	* @function newRow
	* @memberof Widget
	* @instance
	*/
	this.newRow = function() {
		var event;
		if (this.beforeNewRow) {
			event = this.beforeNewRow();
		}
		if (!event || !event.isDefaultPrevented()) {

			var _oldValuesToRepeat = this.getRepeatOnNewValues();
			var row = {};
			if (this.dataSource) {
				this.dataSource.newRow();
				row = this.dataSource.data[this.dataSource.data.length - 1];
			}
			this.setCurrentRow(row);
			this.applyDefaultValue();
			Util.extend(this.currentRow, _oldValuesToRepeat);
			this.edit();
			if (this.onNewRow) {
				this.onNewRow();
			}
		}
	};

	/**
	* Reset the widget fields to it's default values
	* @function applyDefaultValue
	* @memberof Widget
	* @instance
	*/
	this.applyDefaultValue = function() {
		this.fields.forEach(function(field) {
			field.applyDefaultValue();
		});
	};
	/**
	* Removes the row from the widget
	* @function removeRow
	* @memberof Widget
	* @instance
	* @returns {Object} Reloaded data source after removing
	*/
	this.removeRow = function() {
		return this.remove();
	};
	/**
	* Removes the row from the widget
	* @function remove
	* @memberof Widget
	* @instance
	* @returns {Object} Reloaded data source after removing
	*/
	this.remove = function() {
		return this.dataSource.remove(this.currentRow).then((function() {
			return this.reload([],false);
		}).bind(this));
	};
	/**
	* Removes checked rows from the widget
	* @function removeCheckedRows
	* @memberof Widget
	* @instance
	* @returns {Object} Reloaded data source after removing
	*/
	this.removeCheckedRows = function() {
		return this.dataSource.remove(this.dataSource.getCheckedRows()).then((function() {
			return this.reload([],false);
		}).bind(this));
	};
	/**
	* Syncs the data source
	* @function sync
	* @memberof Widget
	* @instance
	* @returns {Promise} Returns a solved promise after updating data source
	*/
	this.sync = function() {
		var event;
		if (this.onSync) {
			event = this.onSync();
		}
		if (!event || !event.isDefaultPrevented()) {
			return this.dataSource.sync();
		} else {
			var deferred = ZHPromise.defer();
			deferred.reject(event);
			return deferred.promise;
		}
	};
	/**
	* Set the current row
	* @function setCurrentRow
	* @memberof Widget
	* @instance
	* @param {Object} row - New row to be set
	* @returns {Promise} Returns a solved promise after updating data source
	*/
	this.setCurrentRow = function(row) {
		var event;
		if (this.beforeMoveRow) {
			event = this.beforeMoveRow();
		}
		if (!event || !event.isDefaultPrevented()) {
			this.currentRow = row;
			if (this.currentRow && this.dataSource && this.dataSource.data) {
				this.currentRow.__index = this.dataSource.data.indexOf(row);
			}
			if (templateManager.project.showDescriptionOrder) {
				this.computeDescriptionOrder();
			}
			if (this.afterMoveRow) {
				this.afterMoveRow();
			}
		}
		return event;
	};
	/**
	* Computes description order
	* @function computeDescriptionOrder
	* @memberof Widget
	* @instance
	*/
	this.computeDescriptionOrder = function() {
		this._descriptionFields = this._descriptionFields || this.fields.filter(function(field) {
				return !Util.isUndefined(field.descriptionOrder);
			})
			.sort(function(a, b) {
				return a.descriptionOrder - b.descriptionOrder;
			});
		if (this._descriptionFields.length === 0 && this.showDescriptionOrder !== false) {
			ZhLogger.warning('No fields with descriptionOrder property were found - Widget: ' + this.name);
			this.descriptionOrder = "descriptionOrder not found";
		} else {
			this.descriptionOrder = this._descriptionFields.map(function(field) {
				return field.modelValue();
			}).join(' - ');
		}
	};
	/**
	* Gets a specific field from the widget
	* @function getField
	* @memberof Widget
	* @instance
	* @param {String} name - Name of the field to be picked
	* @returns {Field} Specified field
	*/
	this.getField = function getField(name) {
		return this.fields.filter(function(item) {
			return item.name === name;
		}).shift();
	};
	/**
	* Gets a specific action from the widget
	* @function getAction
	* @memberof Widget
	* @instance
	* @param {String} name - Name of the action to be picked
	* @returns {Action} Specified action
	*/
	this.getAction = function getAction(name) {
		return this.actions.filter(function(item) {
			return item.name === name;
		}).shift();
	};
	/**
	* Gets all the actions from the widget
	* @function getActions
	* @memberof Widget
	* @instance
	* @returns {Object} Object with actions of the widget
	*/
	this.getActions = function() {
		var actions = [];
		actions = actions.concat(this.actions);
		if (this.widgets) {
			this.widgets.forEach(function(widget) {
				actions = actions.concat(widget.getActions());
			});
		}
		return actions;
	};

	/**
	* Validates widget
	* @function isValidWithMessage
	* @memberof Widget
	* @instance
	* @param {Function} callback - Callback function to show the error message in the Screen. ScreenService.alertNotification for default
	* @returns {Boolean} True if widget is valid
	*/
	this.isValidWithMessage = function(callback) {
		callback = callback || ScreenService.alertNotification.bind(ScreenService);

		this.isValid(function(invalidFieldsArray) {
			invalidFieldsArray.forEach(function(invalidFieldErros) {
				invalidFieldErros.forEach(function(fieldError) {
					if(!fieldError.valid){
						var message = "Error in field '" + fieldError.fieldName + "' with message - " + fieldError.errorMessage + ".\n";
						callback(message);
					}
				});
			});
		});
	};

	/**
	* Validates widget
	* @function isValid
	* @memberof Widget
	* @instance
	* @param {Function} callback - Callback function to receive the array with the invalid fields
	* @returns {Boolean} True if widget is valid
	*/
	this.isValid = function(callback) {
		var invalidFields = [];

		this.fields.forEach(function(field) {
			if (field.validate) {
				var result;
				field.validate(field.value(),function(errorsResult) {
					result = errorsResult;
				});

				if (result) {
					invalidFields.push(result);
				}
			}
		});

		if(callback){
			callback(invalidFields);
		}

		if (this.currentRow.$error) {
			return Object.keys(this.currentRow.$error).filter((function(key) {
				return this.currentRow.$error[key].length > 0;
			}).bind(this)).length === 0;
		} else {
			return true;
		}
	};

	/**
	* Unselect rows
	* @function unSelectAll
	* @memberof Widget
	* @instance
	* @param {String} rows - Rows to be unselected
	* @returns {Event} After unselecting all event
	*/
	this.unSelectAll = function(rows) {
		var event;
		if (this.beforeUnselectAll) {
			event = this.beforeUnselectAll({data: rows});
		}
		if (!event || !event.isDefaultPrevented()) {
			this.dataSource.unSelectAll(rows);
			if (this.afterUnselectAll) {
				this.afterUnselectAll({data: rows});
			}
		}
		return event;
	};
	/**
	* Select rows
	* @function selectAll
	* @memberof Widget
	* @instance
	* @param {String} rows - Rows to be selected
	*/
	this.selectAll = function(rows) {
		var event;
		if (this.beforeSelectAll) {
			event = this.beforeSelectAll({data: rows});
		}
		if (!event || !event.isDefaultPrevented()) {
			this.dataSource.selectAll(rows);
			if (this.afterSelectAll) {
				this.afterSelectAll({data: rows});
			}
		}
	};
	/**
	* Get the last selected row
	* @function getLastSelectedRow
	* @memberof Widget
	* @instance
	* @returns {Object} Selected rows
	*/
	this.getLastSelectedRow = function() {
		return this.selectedRow;
	};
	/**
	* Get checked rows
	* @function getCheckedRows
	* @memberof Widget
	* @instance
	* @returns {Object} Checked rows
	*/
	this.getCheckedRows = function() {
		return this.dataSource.getCheckedRows();
	};
	/**
	* Update checked rows
	* @function updateCheckedRows
	* @memberof Widget
	* @instance
	*/
	this.updateCheckedRows = function() {
		if (this.dataSource) {
			this.dataSource.updateCheckedRows();
		}
		if (this.container) {
			this.container.updateCheckedRows();
		}
		if (this.widgets) {
			this.widgets.forEach(function(widget) {
				widget.updateCheckedRows();
			});
		}
	};
	/**
	* Verifies if widget has rows checked without using cached values
	* @function _hasCheckedRowsWithoutCache
	* @memberof Widget
	* @instance
	* @returns {Boolean} True if widget has checked rows
	*/
	this._hasCheckedRowsWithoutCache = function() {
		return (this.dataSource && this.dataSource.checkedRows && this.dataSource.checkedRows.length > 0);
	};
	/**
	* Verifies if widget has checked rows
	* @function _hasCheckedRows
	* @memberof Widget
	* @instance
	* @returns {Boolean} True if widget have any rows checked
	*/
	this._hasCheckedRows = function() {
		var checkedRows = [];
		if (this.dataSource) {
			checkedRows = this.dataSource.getCheckedRows();
		}
		return checkedRows && checkedRows.length > 0;
	};
	/**
	* Verifies if current widget owns a widget with checked rows
	* @function _hasChildWithCheckedRows
	* @memberof Widget
	* @instance
	* @returns {Boolean} True if one of child widgets has rows checked
	*/
	this._hasChildWithCheckedRows = function() {
		return this.widgets && this.widgets.filter(function(widget) {
			return widget.hasCheckedRows();
		}).length > 0;
	};
	/**
	* Verifies if widget or any of his child has checked rows
	* @function hasCheckedRows
	* @memberof Widget
	* @instance
	* @returns {Boolean} True if widget have any rows checked
	*/
	this.hasCheckedRows = function() {
		return this._hasCheckedRows() || this._hasChildWithCheckedRows();
	};

	/**
	* Enables selection of multiple items in the widget
	* @function enableMultiSelect
	* @memberof Widget
	* @instance
	*/
	this.enableMultiSelect = function() {
		this.allowMultiSelect = true;
	};

	/**
	* Disables selection of multiple items in the widget
	* @function disableMultiSelect
	* @memberof Widget
	* @instance
	*/
	this.disableMultiSelect = function() {
		this.allowMultiSelect = false;
	};

	/**
	* Verifies if floating control is visible
	* @function isFloatingControlVisible
	* @memberof Widget
	* @instance
	* @returns {Boolean} True if widget have a visible floating control
	*/
	this.isFloatingControlVisible = function isFloatingControlVisible() {
		return this.floatingControl && floatingControlAllowsCurrentMode(this.floatingControl.visibleModes);
	};

	function floatingControlAllowsCurrentMode(visibleModes) {
		return !visibleModes || (Util.isArray(visibleModes) && ~visibleModes.indexOf(self.getMode()));
	}
	/**
	* To interface Actionable - Returns true
	* @function addictionalFilterAction
	* @memberof Widget
	* @instance
	* @returns {Boolean} Always true
	*/
	this.addictionalFilterAction = function() {
		return true;
	};

	/**
	* Retrieves the actions of the widget
	* @function getActionsToFilter
	* @memberof Widget
	* @instance
	* @returns {Array} Widget actions
	*/
	this.getActionsToFilter = function() {
		return this.actions;
	};

	/**
	* Empty method
	* @function prepareActions
	* @memberof Widget
	* @instance
	*/
	this.prepareActions = function() {};

	var _setDefaultValuesField = (function(widget) {
		Util.extend(widget, DEFAULT_VALUES);
		Util.extend(widget, new Modeable());
		Util.extend(widget, new Actionable());
		Util.extend(widget, new Templatable());
		widget.formTemplate = widget.templateToForm;
	})(this);
};